;;;
;;; enumconst.scm
;;; Handy for defining flags and constants.
;;;
;;; Copyright (C) 1998-1999, Ivan Demakov.
;;;
;;; Permission to use, copy, modify, and distribute this software and its
;;; documentation for any purpose, without fee, and without a written
;;; agreement is hereby granted, provided that the above copyright notice
;;; and this paragraph and the following two paragraphs appear in all copies.
;;; Modifications to this software may be copyrighted by their authors
;;; and need not follow the licensing terms described here, provided that
;;; the new terms are clearly indicated on the first page of each file where
;;; they apply.
;;;
;;; IN NO EVENT SHALL THE AUTHORS OR DISTRIBUTORS BE LIABLE TO ANY PARTY
;;; FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
;;; INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
;;; DOCUMENTATION, EVEN IF THE AUTHORS HAS BEEN ADVISED OF THE
;;; POSSIBILITY OF SUCH DAMAGE.
;;;
;;; THE AUTHORS AND DISTRIBUTORS SPECIFICALLY DISCLAIMS ANY WARRANTIES,
;;; INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
;;; AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
;;; ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAS NO OBLIGATIONS
;;; TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
;;;
;;;
;;; Author:        Ivan Demakov <demakov@users.sourceforge.net>
;;; Creation date: Tue May 12 18:54:53 1998
;;; Last Update:   $Author: demakov $ $Date: 2000/07/25 19:07:17 $
;;;
;;;
;;; $Id: enumconst.scm,v 1.1.1.1 2000/07/25 19:07:17 demakov Exp $
;;;


;;; (define-enum-constant "errno" intr 9) => (define errno/intr 9)

(define-syntax define-enum-constant
  (lambda (x)
    (define (construct-name templ set elt)
      (let* ((s (syntax-object->datum set))
	     (a (cond ((string? s) s)
		      ((symbol? s) (symbol->string s))
		      (else (syntax-error set "Enum constant base must be symbol or string"))))
	     (b (cond ((identifier? elt)
		       (symbol->string (syntax-object->datum elt)))
		      (else (syntax-error elt "Enum constant ident must be symbol")))))
	(datum->syntax-object templ
			      (string->symbol (string-append a "/" b)))))
    (syntax-case x ()
      ((_ set elt val)
       (with-syntax ((name (construct-name #'_ #'set #'elt)))
	 #'(define name val))))))


;;; (define-enum-constants fruit (apple 1) (orange 2))
;;; => (begin (define-enum-constant fruit apple  1)
;;;           (define-enum-constant fruit orange 2))

(define-syntax define-enum-constants
  (syntax-rules ()
    ((define-enum-constants set (elt val) ...)
     (begin (define-enum-constant set elt val) ...))))


;;; End of code
